#!/bin/bash

QUERYRESULT=""
VERSION=""
DEFVERSION="1.0.0"
RELEASE=""
DEFRELEASE="1"
DRVPACKAGENAME="mvwam"
_WPWD_=`pwd`
TMPBLDDIR=/tmp/MVBUILD.$$
MVWAMDIR=`pwd`
MVWAMDRIV=$MVWAMDIR/driver
MVWAMTOOLS=$MVWAMDRIV/tools
WAMNAME=""

QUERYVERSION="Driver code version"
QUERYRELEASE="Driver code release"

#Errors
errorStatus=0
ERRORNOTROOT="Error: you need to be root to run this program"
ERRORNOTROOTCODE=43
ERRORVERSIONEMPTY="Error: version string entered was empty"
ERRORVERSIONEMPTYCODE=44
ERRORMAKECLEANERROR="Error: make clean failed"
ERRORMAKECLEANERRORCODE=45
ERRORASSERT="ERROR: Unknown error occured"
ERRORASSERTCODE=46
ERRORRPMBUILDFAIL="Error: rpmbuild failed"
ERRORRPMBUILDFAILCODE=47


#list fo files to cleanup
FILESTOCLEAN="./buildout/${PKGNAME} /usr/src/redhat/RPMS/x86_64/${PKGNAME}-*.x86_64.rpm /usr/src/redhat/SOURCES/${PKGNAME}.tar.gz /usr/src/redhat/SRPMS/${PKGNAME}-*.src.rpm"

parseArgs()
{
    while getopts c: OPT
    do
	case "$OPT" in
            c)
		CONFIGFILE=$OPTARG
		;;
	    
            *)
		;;
	esac
    done
}


ASSERT()
{
    if [ $1 -ne 0 ]
	then
	echo $ERRORASSERT
	exit $ERRORASSERTCODE
    fi
}

checkIfRoot()
{
    _lID=`id -u`
    if [ "S$_lID" != "S0" ]
	then
	processError ERRORNOTROOT
    fi
}


processError(){

    if [ $errorStatus -eq 0 ]
    then
	eval errorStatus=\$${1}CODE 
	eval echo \$${1}  >&2
    fi
}

queryUser()
{
    _lString=$1
    printf  "$_lString : "
    read line
    QUERYRESULT=$line
}

#main
#check if root
checkIfRoot

# create the temporary build directory
if [ ! -d ${TMPBLDDIR} ]
    then
        mkdir ${TMPBLDDIR}
        ASSERT $?
fi

parseArgs "$@"

#if a config was specified use it
if [ "S$CONFIGFILE" != "S" ]
then
    . $CONFIGFILE
fi

# 2) ask for the version
if [ $errorStatus -eq 0 -a "S$VERSION" == "S" ]
    then
    queryUser "$QUERYVERSION"
    VERSION=${QUERYRESULT:-${DEFVERSION}}
  
fi

# 2) ask for the release
if [ $errorStatus -eq 0 -a "S$RELEASE" == "S" ]
    then
    queryUser "$QUERYRELEASE"
    RELEASE=${QUERYRESULT:-${DEFRELEASE}}  
fi


# 3) make clean
if [ $errorStatus -eq 0 ]
    then
    ( cd $MVWAMDRIV; make clean )
    if [ $? -ne 0 ]
	then
	processError ERRORMAKECLEANERROR
    fi
fi


# copy everything to temporary build dir
if [  $errorStatus -eq 0 ]
    then
   
    PKGNAME="${DRVPACKAGENAME}-${VERSION}"
 
    echo "cp -r  $MVWAMDIR ${TMPBLDDIR}/${PKGNAME}"
    cp -r  $MVWAMDIR ${TMPBLDDIR}/${PKGNAME}
    ASSERT $?
fi

# 7) create the tar.gz file
if [ $errorStatus -eq 0 ]
then
    rm ${TMPBLDDIR}/${PKGNAME}.tar*

    ( cd ${TMPBLDDIR}/; tar -cvf  ${PKGNAME}.tar ${PKGNAME} )
    ASSERT $?

    gzip ${TMPBLDDIR}/${PKGNAME}.tar
    ASSERT $?

    cp ${TMPBLDDIR}/${PKGNAME}.tar.gz /usr/src/redhat/SOURCES
    ASSERT $?

fi


# 9) Build the RPM 
if [  $errorStatus -eq 0 ]
    then
    rm ${MVWAMDIR}/rpmbuild/${PKGNAME}.spec

    echo "cat ${MVWAMDIR}/rpmbuild/${DRVPACKAGENAME}.spec | sed s/^Version:.*/Version:${VERSION}/g | sed s/^Release:.*/Release:${RELEASE}/g >  ${MVWAMDIR}/buildout/${PKGNAME}.spec"
    cat ${MVWAMDIR}/rpmbuild/${DRVPACKAGENAME}.spec | sed s/^Version:.*/Version:${VERSION}/g | sed s/^Release:.*/Release:${RELEASE}/g >  ${MVWAMDIR}/buildout/${PKGNAME}.spec
    ASSERT $? 

    rpmbuild -ba  ${MVWAMDIR}/buildout/${PKGNAME}.spec
    if [ $? -ne 0 ]
	then
	processError ERRORRPMBUILDFAIL
    fi
    
fi



# 10) copy the rpm into local dir
if [ $errorStatus -eq 0 ]
then
    cp /usr/src/redhat/RPMS/`uname -m`/${PKGNAME}*.`uname -m`.rpm    ${MVWAMDIR}/buildout
    ASSERT $? 
fi



# 11) cleanup SOURCES , RPMS , SPRMS
FILESTOCLEAN="/usr/src/redhat/RPMS/`uname -m`/${PKGNAME}-*.`uname -m`.rpm /usr/src/redhat/SOURCES/${PKGNAME}.tar.gz /usr/src/redhat/SRPMS/${PKGNAME}-*.src.rpm  /usr/src/redhat/BUILD/${PKGNAME}* ${MVWAMDIR}/buildout/*.spec"

rm -rf $FILESTOCLEAN  >/dev/null 2>&1

# 12) tell the user where to find the built RPM..
if [ $errorStatus -eq 0 ]
then
    echo "RPM successfully built! at >> ${MVWAMDIR}/buildout/${PKGNAME}-${RELEASE}.`uname -m`.rpm"
fi

